%differentiate
% Function which computes, and plots, the gradient of a Mathematical
% function. It also plots the tangent and normal curves to a specified point
% on the curve.
%
% LAST UPDATED by Andy French April 2016

function differentiate

%Number of graph evaluation points
N = 1000;

%Fontsize for graph
fsize = 16;

%

%Display opening message
disp(' ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp('  Differentiation based on cubic splines')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp(' ')

%Get inputs from user

%Define function y = f(x) to differentiate
disp(' Define function y = f(x) to differentiate' )
f = input(' > f(x) = ','s'); 
if isempty(f)
    disp(' Default function is f(x) = (4-x)*x^2');
    f = '(4-x)*x^2';
end
disp(' ')

%Define x coordinate to evaluate derivative and plot tangent and normal 
disp(' Define x coordinate to evaluate derivative and plot tangent and normal ')
x0 = input(' > x0 = '); 
if isempty(x0)
    disp(' x0 = 2' );
    x0 = 2;
end

%Define graph x limits
disp(' ')
disp(' Define graph x limits in the form [xmin,xmax] ')
xlimits = input(' > [xmin,xmax] = '); 
if isempty(xlimits)
    disp(' Default graph xlimits are [-10,10]' );
    xlimits = [-10,10];
end

%Define graph y limits
disp(' ')
disp(' Define graph y limits in the form [ymin,ymax] ')
ylimits = input(' > [ymin,ymax] = '); 
if isempty(ylimits)
    disp(' Default graph ylimits are [-10,10]' );
    ylimits = [-10,10];
end

%Generate x,y data
x = x0;
eval(['y0 = ',F(f),';']);
x = linspace( xlimits(1),xlimits(2),N);
eval(['y = ',F(f),';']);

%Differentiate data
dy_by_dx = differentiator( x, y, x );
dy_by_dx0 = differentiator( x, y, x0 );

%Dedtermine equation of tangent and normal through (x0,y0)
yN = y0 -( x - x0)/dy_by_dx0;
yT = y0 +( x - x0)*dy_by_dx0;

%Plot graphs
disp(' ')
disp(' Plotting graph...')
disp(' ')
figure('color',[ 1 1 1],'name','differentiate')
plot(x,y,'b');
axis equal
ylim(ylimits);
xlim(xlimits);
hold on;
plot(x,yN,'r');
plot(x,yT,'g');
plot(x, dy_by_dx, 'm--');
grid on
box on
plot( xlimits, [0 0],'linewidth', 1,'color',[0,0,0] );
plot( [0,0], ylimits,'linewidth', 1,'color',[0,0,0] );
set(gca,'fontsize',fsize)
xlabel('x','fontsize',fsize)
ylabel('y','fontsize',fsize)
title(['Gradient of f(x) = ',f,' is ',num2str(dy_by_dx0),...
    ' at ','(',num2str(x0),',',num2str(y0),')'],...
  'fontsize',fsize)
print( gcf, ['gradient ',strrep(datestr(now),':','-')],'-dpng','-r300' )
disp(' Graph plotted.')
disp(' ');

%%

%differentiator
% Differentiation function using cubic splines.
%
% Syntax: dy_by_dx = differentiate( xx, yy, x )
%
% xx, yy     Samples from function y(x) which is being differentiated. These
%            must be the same dimensions and have at least three elements each.
% x          Point on y(x) at which the gradient is returned. This can be an array of any
%            dimensions.
%
% dy_by_dx   Output of gradient of y(x) corresponindg to array x
%
% The function is based on the code written on pages 292 and 294
% of "Mastering Matlab 6" by Duane Hanselman & Bruce Littlefield.

function dy_by_dx = differentiator( xx, yy, x )

%Check inputs are the right size
if ( numel(xx) == numel(yy) ) && ( numel(xx) >= 3 )
    
    %Sort xx in ascending order
    [xx,i] = sort(xx);
    yy = yy(i);
    
    %Compute piecewise polynomial based upon cubic splines.
    pp=spline(xx,yy);
    %pp = pchip(xx,yy);  %Alternative Hermite polynomial fit. Apparently this
    %is slightly better at preserving the shaoe of the function near
    %discontinuities.
    
    %Take apart piecewise polynomial
    [br,co,npy,nco]=unmkpp(pp);
    
    %Scale factors for differentiation
    sf=nco-1:-1:1;
    
    %Derivative coefficients
    dco=sf(ones(npy,1),:).*co(:,1:nco-1);
    
    %Build pp form for derivative
    ppd=mkpp(br,dco); 
    
    %Generate the values of dy_by_dx by evaluating the piecewise polynomial
    dy_by_dx = ppval(ppd,x);
    
else
    dy_by_dx = NaN(size(x));
end

%%

%F
% Function which adds . to enable elementwise matrix multiplication of
% variables in string s
function s = F(s)
s = strrep(s,'^','.^');
s = strrep(s,'*','.*');
s = strrep(s,'/','./');

%End of code