%integrate
% Function which computes the definite integral of a specified function.
% This is displayed as a graph of the function, with the area being
% integrated shown as a semi-transparent green patch.
%
% LAST UPDATED by Andy French. July 2014.

function integrate

%Number of graph evaluation points
N = 1000;

%Fontsize for graph
fsize = 16;

%

%Display opening message
disp(' ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp('  Integration based on cubic splines')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp(' ')

%Get inputs from user

%Define function y = f(x) to integrate
disp(' Define function y = f(x) to integrate' )
f = input(' > f(x) = ','s'); 
if isempty(f)
    disp(' Default function is f(x) = (4-x)*x^2');
    f = '(4-x)*x^2';
end
disp(' ')

%Define limits of integration
disp(' Define integration x limits in the form [xmin,xmax] ')
ixlimits = input(' > [xmin,xmax] = '); 
if isempty(ixlimits)
    disp(' Default integration xlimits are [0,4]' );
    ixlimits = [0,4];
end

%Define graph limits
disp(' ')
disp(' Define graph x limits in the form [xmin,xmax] ')
xlimits = input(' > [xmin,xmax] = '); 
if isempty(xlimits)
    disp(' Default graph xlimits are [-3,6]' );
    xlimits = [-3,6];
end

%

%Generate x,y data
x = linspace( xlimits(1),xlimits(2),N);
eval(['y = ',F(f),';']);

%Integrate data
I = integrator( x, y, ixlimits(1), ixlimits(2) );

%Define x,y coordinates of green patch which defines integral area
xi = x( ( x>=ixlimits(1) ) & ( x<=ixlimits(2) ) );
yi = y( ( x>=ixlimits(1) ) & ( x<=ixlimits(2) ) );
xi = [ ixlimits(1),xi, ixlimits(2), ixlimits(1) ];
yi = [ 0,yi, 0, 0 ];

%Plot graph
disp(' ')
disp(' Plotting graph...')
disp(' ')
figure('color',[ 1 1 1],'name','integrate')
p = patch(xi,yi,'g');
set(p,'facealpha',0.5);
hold on;
plot(x,y);
axis tight
grid on
box on
plot( xlimits, [0 0],'linewidth', 1,'color',[0,0,0] );
ylimits = get(gca,'ylim');
ylimits = round(ylimits);
ylim(ylimits);
plot( [0,0], ylimits,'linewidth', 1,'color',[0,0,0] );
set(gca,'fontsize',fsize)
xlabel('x','fontsize',fsize)
ylabel('y','fontsize',fsize)
title(['Integral of f(x) = ',f,' between ',...
    num2str(ixlimits(1)),' and ',num2str(ixlimits(2)),...
    ' = ',num2str(I)],...
  'fontsize',fsize)
print( gcf, ['integral ',strrep(datestr(now),':','-')],'-dpng','-r300' )
disp(' Graph plotted.')
disp(' ');

%%

%integrator
% Integration function using cubic splines.
%
% Syntax: I = integrator( xx, yy, x0, x )
%
% xx, yy     Samples from function y(x) which is being integrated. These
%            must be the same dimensions and have at least three elements each.
% x0         Starting point (i.e. lower limit) of the integral
% x          Upper limit of the integral. This can be an array of any
%            dimensions.
%
% I          Output of integral corresponindg to array x
%
% The function is based on the code written on pages 292 and 294
% of "Mastering Matlab 6" by Duane Hanselman & Bruce Littlefield.

function I = integrator( xx, yy, x0, x )

%Check inputs are the right size
if ( numel(xx) == numel(yy) ) && ( numel(xx) >= 3 )
    
    %Sort xx in ascending order
    [xx,i] = sort(xx);
    yy = yy(i);
    
    %Compute piecewise polynomial based upon cubic splines.
    pp=spline(xx,yy);
    
    %Take apart piecewise polynomial
    [br,co,npy,nco] = unmkpp(pp);
    
    %Scale factors for integration
    sf=nco:-1:1;
    
    %Integral coefficients
    ico=[co./sf(ones(npy,1),:) zeros(npy,1)];
    
    %Integral spline has higher order
    nco=nco+1;
    
    %Set integration constant of all splines to be zero
    ico(1,nco)=0;
    
    %Find constant terms in polynomials
    for k=2:npy
        ico(k,nco)=polyval(ico(k-1,:),br(k)-br(k-1));
    end
    
    %Build pp form for integral
    ppi=mkpp(br,ico);
    
    %Evaluate definite integral between limits of x0 to x
    I = ppval(ppi,x) - ppval(ppi,x0);
    
else
    I = NaN(size(x));
end

%%

%F
% Function which adds . to enable elementwise matrix multiplication of
% variables in string s
function s = F(s)
s = strrep(s,'^','.^');
s = strrep(s,'*','.*');
s = strrep(s,'/','./');

%End of code