%second_order_solver
% Numeric solver for second order Ordinary Differential Equations
% (ODEs) of the form:
%
% dv/dt = f( v, x, t )
% dx/dt = g( v,x, dv/dt, t )
%
% Note x can be a vector 
% i.e. x(1,:) would be 'x', x(2,:) would be 'y' etc.
% 
%
% LASt UPDATED by Andy French April 2016

function second_order_solver

%Fontsize for graph
fsize = 16;

%

%Display opening message
disp(' ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%% ')
disp('  Second order ODE solver')
disp(' %%%%%%%%%%%%%%%%%%%%%%%% ')
disp(' ')

%Define function dx/dt = g( v,x, dv_by_dt, t )
disp(' Define dx/dt = g( v,x, dv_by_dt, t ) ')
g = input(' > g( v,x,dv_by_dt,t ) = ','s');
if isempty(g)
    disp(' Default function is g( v,x, dv_by_dt, t ) ' );
    g = 'v';
end
disp(' ')

%Define function dv/dt = f( v,x, t )
disp(' Define dv/dt = f( v,x, t ) ')
f = input(' > f( v,x,t ) = ','s');
if isempty(f)
    disp(' Default function is f( v,x,t ) = -9.81*[0;1] - 0.1*v*sqrt(dot(v,v)) ' );
    f = '-9.81*[0;1] - 0.1*v*sqrt(dot(v,v))';
end
disp(' ')

%Define stopping test
disp(' Define stopping test ')
stop_test = input(' > ','s');
if isempty(stop_test)
    disp(' Default stopping test is: x(2)<0 ' );
    stop_test = 'x(2)<0';
end
disp(' ')

%Initial conditions
disp(' Define initial conditions ')
x0 = input(' > x0 = ');
if isempty(x0)
    disp(' Default x0 = [0,0] ' );
    x0 = [0,0];
end
v0 = input(' > v0 = ');
if isempty(v0)
    disp(' Default v0 = 10*[ cos(pi/4), sin(pi/4) ] ' );
    v0 = 10*[ cos(pi/4), sin(pi/4) ];
end
t0 = input(' > t0 = ');
if isempty(t0)
    disp(' Default t0 = 0 ' );
    t0 = 0;
end
disp(' ')

%Define timestep
disp(' Define timestep ')
dt = input(' > dt = ');
if isempty(dt)
    disp(' Default timestep = 0.01' );
    dt = 0.01;
end
disp(' ')

%Define end time
disp(' Define end time ')
tend = input(' > tend = ');
if isempty(tend)
    disp(' Default tend = 15' );
    tend = 15;
end
disp(' ')

%Define solver method
disp(' Define solver Method ')
disp(' Option 1: Euler')
disp(' Option 2: Verlet')
disp(' Option 3: Runge-Kutta')
disp(' ')
option = input(' > solver option = ','s');
if ( strcmp( option,'Euler' )==0 ) && ( strcmp( option,'Verlet' )==0 )
    disp(' Default solver option is Runge-Kutta' );
    option = 'Runge-Kutta';
end
disp(' ')

%

%Solve
[dv_by_dt,v,dx_by_dt,x,t,tstop] = ...
    solver2( v0, x0, t0, tend, dt, f, g, stop_test, option );

%Plot solution
figure('color',[ 1 1 1 ])

%Get dimensions and determine whether 1D,2D or 3D problem
dim = size(x0);
if dim(2)==1
    plot( t, x); hold on
    plot( t(1),x(1),'g*','markersize',fsize);
    plot( t(end),x(end),'r*','markersize',fsize);
    xlabel('t','fontsize',fsize)
    ylabel('x','fontsize',fsize)
elseif dim(2)==2;
    plot( x(1,:), x(2,:) ); hold on
    plot( x(1,1),x(2,1),'g*','markersize',fsize);
    plot( x(1,end),x(2,end),'r*','markersize',fsize);
    xlabel('x','fontsize',fsize)
    ylabel('y','fontsize',fsize)
    axis equal
elseif dim(2)==3
    plot3( x(1,:), x(2,:), x(3,:) ); hold on
    plot3( x(1,1),x(2,1),x(3,1),'g*','markersize',fsize);
    plot3( x(1,end),x(2,end),x(3,end),'r*','markersize',fsize);
    xlabel('x','fontsize',fsize)
    ylabel('y','fontsize',fsize)
    zlabel('y','fontsize',fsize)
    axis vis3d
end
title({'Numeric solution to second order ODE',['dx/dt = ',g],['dv/dt = ',f],...
    ['Stopped at t = ',num2str(tstop)] },'fontsize',fsize);
set(gca,'fontsize',fsize)
grid on

%Save .png file
print( gcf, ['second order solver ',strrep(datestr(now),':','-')],'-dpng','-r300' )

%%

%solver2
% Numeric solver for second order Ordinary Differential Equations (ODEs) of the
% form:
%
% dv/dt = f( v, x, t )
% dx/dt = g( v,x, dv/dt, t )
%
% LAST UPDATED by Andy French October 2013
%
% Syntax: [dv_by_dt,v,dx_by_dt,x,t] = solver2( v0, x0, t0, tend, dt, F, G, method )
%
% x0            Initial value of x. This can be a scalar, vector or array.
% t0            Initial value of time variable t
% tend          Final value of time variable
% dt            Time step (which is constant)
% F             String describing f( v,x, t ). Note the dimensions of this must match
%               those of input x. i.e. if x is a 1x10 vector then so if f.
%               e.g. 'y = v + x + cos(t)'.  Must start with 'y = ...'
% G             String describing g( v,x, dv/dt, t ).
%               e.g. 'y = v - x + dv_by_dt + t';
% stop_test     String describing a stop test. Can use 'x', 'v', 't' and
%               'dx_by_dt' and 'dv_by_dt' but must return either a 0 or 1. i.e. take care if x,t etc are
%               not scalars. e.g. stop_test = 'x<=0'
% method        Choose from three methods 'Runge-Kutta', 'Verlet', 'Euler'.
%               Default is 'Runge-Kutta.'
%
% dv_by_dt,v,dx_by_dt,x,t
%               Solutions to ODE. If not a scalar, then this will have one
%               dimension extra than x0, corresponding to different time steps.
% t             Array of times t(n+1) = t(n) + dt. Same dimensions as x.
% tstop         Stopping time. This will be t(end) unless the stopping test
%               has been passed.

function [dv_by_dt,v,dx_by_dt,x,t,tstop] = solver2( v0, x0, t0, tend, dt, F, G, stop_test, method )

%Determine initial values of t,x, dx_by_dt and the store of these for all
%time values computed by the solver. Note this will add an extra dimension
%to whatever sized array the input x0 is.
[N,t,x,dx_by_dt,v,dv_by_dt,t_store,x_store,dx_by_dt_store,v_store,dv_by_dt_store] =...
    initial_values( t0, tend, dt, x0, v0, F, G );

%Initially set tstop to be tend
tstop = tend;

%Apply ODE solver iteratively
tt = t0;
if N >=2
    for n=2:N
        
        %Check stop test
        test_result = eval( stop_test );
        if test_result == 0
            
            %Update t, dv_by_dt and dx_by_dt
            t = t + dt;
            tt = tt + dt;
            dv_by_dt = f( v, x, t, F );
            dx_by_dt = g( v, x, dv_by_dt, t, G );
            
            %Update x,v via a numerical method
            if strcmp(method,'Euler')
                
                %First order Euler method
                x = x + dt * dx_by_dt;
                v = v + dt * dv_by_dt;
                
            elseif strcmp(method,'Verlet')
                
                %Second order Verlet method (This only really works when dx/dt = v and
                %corresponds to constant acceleration motion between time steps)
                v = v + dt * dv_by_dt;
                x = x + v*dt + 0.5 * dt * dt * dv_by_dt;
                
            else
                
                %4th order Runge-Kutta method for v
                k1 = f( v, x, t, F );
                k2 = f( v + 0.5*dt*k1, x, t + 0.5*dt, F );
                k3 = f( v + 0.5*dt*k2, x,  t + 0.5*dt, F );
                k4 = f( v + dt*k3, x, t + dt, F );
                
                v = v + (1/6)*dt* ( k1 + 2*k2 + 2*k3 + k4 );
                
                %4th order Runge-Kutta method for x
                k1 = g( v, x, dv_by_dt, t, G );
                k2 = g( v, x + 0.5*dt*k1, dv_by_dt, t + 0.5*dt, G );
                k3 = g( v, x + 0.5*dt*k2, dv_by_dt, t + 0.5*dt, G );
                k4 = g( v, x + dt*k3, dv_by_dt, t + dt, G );
                
                x = x + (1/6)*dt* ( k1 + 2*k2 + 2*k3 + k4 );
            end
        else
            tstop = tt;
        end
        
        %Store these new values
        dx_by_dt_store = update_store( dx_by_dt_store, n, dx_by_dt );
        x_store = update_store( x_store, n, x );
        dv_by_dt_store = update_store( dv_by_dt_store, n, dv_by_dt );
        v_store = update_store( v_store, n, v );
    end
end

%Set outputs
x = x_store;
v = v_store;
dv_by_dt = dv_by_dt_store;
t = t_store;

%%

%Determine initial values of variables, and initialize variable store
function [N,t,x,dx_by_dt,v,dv_by_dt,t_store,x_store,dx_by_dt_store,v_store,dv_by_dt_store] =...
    initial_values( t0, tend, dt, x0, v0, F, G )

%Make sure x0 is a column vector, if x0 is indeed a vector.
if isvector(x0)==1
    dim = size(x0);
    if dim(1) == 1;
        x0 = x0.';
    end
end

%Make sure v0 is a column vector, if v0 is indeed a vector.
if isvector(v0)==1
    dim = size(v0);
    if dim(1) == 1;
        v0 = v0.';
    end
end

%Determine initial time array (this will be updated
%each iteration by the ODE solver
if isscalar(x0)==1
    %Scalar
    t = t0;
elseif isvector(x0);
    %Vector
    t = t0*ones( numdim(x0), 1 );
else
    %Multi-dimensional array
    t = t0*ones( numdim(x0) );
end

%Initialize x, v and their derivatives arrays
x = x0;
v = v0;
dv_by_dt = f( v, x, t, F );
dx_by_dt = g( v, x, dv_by_dt, t, G );

%Determine number of time steps and therefore construct time store
tt = t0 : dt : tend;
N = length(tt);
t_store = update_store( ones( [ numdim(x0), N ] ), 1, t0 );
for n=1:N
    t_store = update_store( t_store, n, tt(n) );
end

%Initialize store of x, v and dv_by_dt
dv_by_dt_store = update_store( zeros( [ numdim(x0), N ] ), 1, dv_by_dt );
v_store = update_store( zeros( [ numdim(x0), N ] ), 1, v0 );
dx_by_dt_store = update_store( zeros( [ numdim(x0), N ] ), 1, dx_by_dt );
x_store = update_store( zeros( [ numdim(x0), N ] ), 1, x0 );

%%

%Evaluate f( v, x, t, F )
function y = f( v, x, t, F )
eval( ['y = ',F,';' ] );

%%

%Evaluate g( v, dv_by_dt, x, t, G )
function y = g( v, x, dv_by_dt, t, G )
eval( ['y = ',G,';' ] );

%%

%Add last dimension of multi-dimensional array
% e.g. if x is a 3D array, x = update_store(x,n,xx) is equivalent to x(:,:,n ) = xx(:,:)
function x = update_store(x,n,xx)

%Update differently depending whether xx is scalar, vector or something of
%higher dimensionality.
if numel(numdim(xx))==1
    %Scalar or vector
    x(:,n) = xx(:);
else
    %Multi-dimensional array
    colons = [];
    for n=1:numel(numdim(xx));
        colons = [colons,':,'];
    end
    x_str = ['x(',colons(1:end-1),',n ) = xx(',colons(1:end-1),');'];
    eval(x_str);
end

%%

%numdim
% Number of dimensions function
function N = numdim(x)
if isscalar(x)==1
    %Scalar
    N = 1;
elseif isvector(x)==1
    %Vector
    N = numel(x);
else
    %Array of some sort (matrix or higher dimension)
    N = size(x);
end

%End of code