%mandeltrajgui
% This program firstly determines a visual representation of the Mandelbrot
% set, and then overlays the first few iterations of z using the Mandelbrot
% transform based upon the current location of the mouse. Pressing any key
% will print the current screen.
%
% LAST UPDATED by Andy French April 2016

function mandeltrajgui

%Initialise global array to store GUI data
global d

%Define set parameters

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Number of Mandlebrot iterations for trajectories
d.N = 50;

%Number of iterations for underlay plot
d.N_iter = 20;

%Iteration function. It must be of the form 'z = f(z,z0)'
d.iter_func = 'z = ( z.^2 +z0 )';

%Axes dimensions
d.xmin = -0.7*pi;
d.xmax = 0.3*pi;
d.ymin = -pi/3;
d.ymax = pi/3;

%Number of x,y divisions
d.N_div = 500;

%Fontsize
d.fsize = 18;

%Define colormap
d.cmap_str = 'jet';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Create figure and axes
d.fig = figure('name','Mandelbrot trajectories GUI','color',[1 1 1],...
    'KeyPressFcn',@keypressfunc,'WindowButtonDownFcn',@mouseclickdown,...
    'WindowButtonMotionFcn', @mousedrag,...
    'WindowButtonUpFcn',@mouseclickup,'renderer','painters','numbertitle','off' );
d.ax = axes;

%Set colormap
cmap = colormap( d.cmap_str );

%Create Mandelbrot underlay image (this is much faster than a surface)
x = linspace( d.xmin,d.xmax, 2*d.N_div );
y = linspace( d.ymin,d.ymax, d.N_div );
[xx,yy] = meshgrid(x,y);
z0 = xx+1i*yy;
z = z0;
for n=1:d.N_iter
    eval( [d.iter_func,';'] );
end
f = exp(-abs(z));
f( f < 0.0001 ) = NaN;
im = make_colour_matrix(f,interp_colormap(500,cmap),[1 1 1],[0,1] );
clear xx yy f z z0

%Plot underlay image
image( x,y, im);
view(2);
axis equal
axis tight
grid on;
box on;
hold on;

%Add axis annotation
xlabel('x = Re(z)','fontsize',d.fsize);
ylabel('y = Im(z)','fontsize',d.fsize);
set( d.ax, 'fontsize', d.fsize );
title( 'Mandelbrot z_{n+1}=  z_n^2 + z_0', 'fontsize', d.fsize  )

%Initalize stored variables which correspond to Mandelbrot trajectories
d.draw_mandel_traj = 0;
d.mandel_traj0 = plot3( 0,0,0,'r*','markersize',18,'visible','off' );
d.mandel_traj1 = plot3( 0,0,0,'k*','markersize',10,'visible','off' );
d.mandel_traj2 = plot3( 0,0,0,'k-','linewidth',1,'visible','off' );

%%

%%%% KEYBOARD & MOUSE INTERACTION CALLBACKS %%%%

% e.g. key press handler, mouse click event manager,
% close request function etc

%Key press handler
function keypressfunc(hObject, eventdata, handles)
global d

%Get the string of the current key press
s = get( d.fig,'currentkey');

%Actions, depending on the key pressed...
if ( strcmp(s,'q') == 1 ) || ( strcmp(s,'Q') == 1 )
    %Quit GUI
    close(gcf);
else
    %Print screen
    drawnow;
    print( d.fig, ['Mandel-traj ',strrep( datestr(now),':','-' ),'.png'],'-dpng','-r300' );
end

%%

%This function executes when the mouse is clicked
function mouseclickdown(hObject, eventdata, handles)
global d

%Store 'click point' coordinates (pixels from bottom left of the figure)
cp=get( d.ax,'currentpoint');
x0 = cp(1,1);
y0 = cp(1,2);

%Check 'click point' coordinates are within axes
xv = [ d.xmin, d.xmax, d.xmax, d.xmin ];
yv = [ d.ymin, d.ymin, d.ymax, d.ymax ];

%Overlay first d.N Mandelbrot iterations
if inpolygon( x0, y0, xv, yv ) == 1
    d.draw_mandel_traj = 1;
    [x,y] = mandel_traj( x0,y0, d.N, d.iter_func );
    set( d.mandel_traj1, 'xdata', x(2:end), 'ydata', y(2:end),'zdata',ones(1,d.N-1),'visible','on' );
    set( d.mandel_traj2, 'xdata', x, 'ydata', y,'zdata',ones(size(x)),'visible','on' );
    set( d.mandel_traj0, 'xdata', x0, 'ydata', y0,'zdata',1,'visible','on' );
end

%%

%This function runs when a mouse is dragged following a click event, and
%before the mouse button is released
function mousedrag(hObject, eventdata, handles)
global d

%Store 'click point' coordinates (pixels from bottom left of the figure)
cp=get( d.ax,'currentpoint');
x0 = cp(1,1);
y0 = cp(1,2);

%Check 'click point' coordinates are within axes
ax_pos = get( d.ax, 'position' );
xv = [ d.xmin, d.xmax, d.xmax, d.xmin ];
yv = [ d.ymin, d.ymin, d.ymax, d.ymax ];

%Overlay first d.N Mandelbrot iterations
if ( inpolygon( x0, y0, xv, yv ) == 1 ) && ( d.draw_mandel_traj == 1 )
    [x,y] = mandel_traj( x0,y0, d.N,d.iter_func );
    set( d.mandel_traj1, 'xdata', x(2:end), 'ydata', y(2:end) );
    set( d.mandel_traj2, 'xdata', x, 'ydata', y );
    set( d.mandel_traj0, 'xdata', x0, 'ydata', y0,'zdata',1,'visible','on' );
end
drawnow;

%%

%This function executes when a mouse button is released
function mouseclickup(hObject, eventdata, handles)
global d

%Prevent any further dynamic update during mouse dragging
d.draw_mandel_traj = 0;

%%

%mandeltraj
% Function which computes the next N - 1 complex numbers along a Mandelbrot trajectory
function [x,y] = mandel_traj( x0,y0, N, iter_func )
x = x0*ones(1,N);
y = y0*ones(1,N);
z0 = x0 + 1i*y0;
z = z0;
if N>1
    for n=2:N
        eval( [iter_func,';'] );
        x(n) = real(z);
        y(n) = imag(z);
    end
end

%%

%interp_colormap
% Function which interpolates current colourmap to yield better graduated
% shading. N is number of possible colours.
%
% LAST UPDATED by Andy French 14 September 2008
function cmap = interp_colormap( N, cmap )

%Initialise new colormap
new_map = ones(N,3);

%Get size of current colormap and initalise red,green,blue vectors
dim = size(cmap);
R = ones(1,dim(1));
G = ones(1,dim(1));
B = ones(1,dim(1));
RR = ones(1,N);
GG = ones(1,N);
BB = ones(1,N);

%Populate these with current colormap
R(:) = cmap(:,1);
G(:) = cmap(:,2);
B(:) = cmap(:,3);

%Interpolate to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x );
GG = interp1( 1:dim(1), G, x );
BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:);
new_map(:,2) = GG(:);
new_map(:,3) = BB(:);

%%

%make_colour_matrix
% Function that takes a two dimensional array of intensities
% and converts this into an RGB array for use in the imwrite.m function
%
% Syntax: C=make_colour_matrix(A,cmap,NaN_colour,clim)
%
% A      - M x N array of 'intensities', these should be normalised such as
%          the magnitudes will be in the interval [0,1.
% cmap   - Colour map. i.e.  cmap=colormap('prism')
% NaN_colour
%        - [R,G,B] vector of NaN colours
%
% C      - M x N x 3 RGB matrix. Each page corresponds to RGB colurs
%          (respectively) in the range [0,1];
% clim   - Vector which maps range of A to colourmap range
function C=make_colour_matrix(A,cmap,NaN_colour,clim)

%Get dimensions of colourmap and initialise red, green and blue vectors
cmap_dim=size(cmap);
red=zeros(1,cmap_dim(1));
green=zeros(1,cmap_dim(1));
blue=zeros(1,cmap_dim(1));
red(:)=cmap(:,1);
green(:)=cmap(:,2);
blue(:)=cmap(:,3);

%Define colour scale clim to enable the colormap to be interpolated
cscale=linspace(clim(1),clim(2),cmap_dim(1));

%Initialise output RGB matrix
C=zeros([size(A),3]);

%Determine R,G,B components of C using linear interpolation of the
%colormap. Set NaNs to be the specified NaN colour.
C_red=interp1(cscale,red,A,'linear');
C_red(isnan(C_red)==1)=NaN_colour(1);
C_green=interp1(cscale,green,A,'linear');
C_green(isnan(C_green)==1)=NaN_colour(2);
C_blue=interp1(cscale,blue,A,'linear');
C_blue(isnan(C_blue)==1)=NaN_colour(3);
C(:,:,1)=C_red;
C(:,:,2)=C_green;
C(:,:,3)=C_blue;

% End of code