%mandlesurface
% Function which plots a sequence of surfaces illustrating an iterative complex
% mapping of the argand diagram.
%
% LAST UPDATED by Andy French April 2016

function mandlesurface

%Number of points per axis
N = 500;

%Number of iterations
iter = 1;

%Complex mapping function z = ....
zmap = 'z = z.^2 + z0' ;
% zmap = 'z = z.^2 + z0; z = atan(z)' ;
% zmap = 'z = z.^2 + z0; z = sin(z)' ;
% zmap = ' z = z.^2 + z0; z = asin(z)' ;
% zmap = 'z = z.^2 + z0; z = log(z)' ;
% zmap = 'z = z.^2 + z0; z = conj(z)' ;
% zmap = 'z = z.^2 + z0; z = z.^2;' ;
% zmap = 'z = cos(z)-sin(z)./(cos(z)+sin(z))';
% zmap = 'z = z.^z';
% zmap = 'z = z - 1./z';

%Surface plotting function
zplot = 'exp( -abs(z) )';
%zplot = 'angle(z)';
%zplot = 'normalize(exp( -abs(z) ))';
%zplot = 'log(abs(z))';
%zplot = 'sin(abs(z))'

%Set colour axis limits
cmin = 0;
cmax = 1;

%Extent of argand diagram
xmin = -2.1;
xmax = 0.7;
ymin = -1.2;
ymax = 1.2;
zmin = 0;
zmax = 1;

%Pause between iterations /s
pause_time = 0.2;

%Save indexed images? (This will slow things down a bit)
savepics = 0;

%Fontsize for plots
fsize = 12;

%Print resolution
DPI = 300;

%

%Display message to command window
disp(' ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%')
disp('  Iterative Mandelbrot surface generator ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%')
disp(' ')

%Start clock
t0=tic;

%Create Argand diagram
xx = linspace( xmin, xmax, N );
yy = linspace( ymin, ymax, N );
z = zeros(N,N);
y = zeros(N,N);
x = zeros(N,N);
for n=1:N
    for m =1:N
        z(n,m) = xx(n) + 1i*yy(m) ;
        x(n,m) = xx(n);
        y(n,m) = yy(m);
    end
end
z0=z;

%Create a directory in which plots are stored
dirname = ['mandelsurface ',strrep( datestr(now),':','-' )];
mkdir(dirname);

%Plot surface following transformation
fig = figure('color',[1 1 1],'name','mandelsurface','renderer','opengl','numbertitle','off');
ax = axes;
colormap('prism'), 
interp_colormap(N);
s = get(0,'screensize');
set(fig,'units','pixels','position',s);
for k=1:iter
    
    %Display message to command window
    disp([' Iteration ',num2str(k),' of ',num2str(iter),' ...'])
    
    %Pause to plot and then clear previous axes
    if k>1
        pause(pause_time);
        cla;
    end
    
    %Create surface
    subplot(1,2,1);
    eval( ['surf(x,y,',zplot,');'])
    
    %Annotate figure, set colouring, lighting etc
    set(fig,'name',[' Iteration ',num2str(k),' of ',num2str(iter),' ...']);
    xlabel('x','fontsize',fsize)
    ylabel('y','fontsize',fsize)
    zlabel(['z = ',zplot],'fontsize',fsize)
    set(gca,'fontsize',fsize)
    caxis([cmin,cmax])
    shading interp;
    axis vis3d
    axis equal
    view([-35,64]);
    xlim([xmin,xmax])
    ylim([ymin,ymax])
    zlim([zmin,zmax])
    alpha(0.9);
    lighting phong
    camlight
    zoom(1.3);

    %Create 2D view
    subplot(1,2,2);
    eval( ['pcolor(x,y,',zplot,');'])
    caxis([cmin,cmax])
    
    %Annotate figure, set colouring, lighting etc
    %xlabel('real numbers x')
    %ylabel('complex numbers i*y')
    zlabel(['z = ',zplot],'fontsize',fsize)
     set(gca,'fontsize',fsize)
    caxis([cmin,cmax])
    title( ['Mandlebrot surface: iteration ',num2str(k)],'fontsize',fsize );
    shading interp;
    axis equal
    view(2);
    xlim([xmin,xmax])
    ylim([ymin,ymax])
    drawnow
    
    %Save an indexed PNG file of the surface for this iteration
    if savepics == 1
        print( gcf,[dirname,'\mandelsurface ',...
            leadingzero(k,3) ],'-dpng',['-r',num2str(DPI)]);
    end
    
    %Update argand diagram
    eval( [zmap,';'] )
end

%Always save last image
print( gcf,[dirname,'\mandelsurface ',...
    leadingzero(k,3) ],'-dpng',['-r',num2str(DPI)]);
close(gcf)

%Display completion message
disp(' ')
disp([' All Mandelbrot surfaces generated in ',num2str(toc(t0)/60),' minutes.']);
disp(' ');

%%

%Indexed images of name 00001, 00002 etc
function outstring = leadingzero(X,N)
outstring = [strrep(blanks(N-1 - fix(log10(X))),...
    ' ','0'),num2str(X,3)];

%%

%Normalize magnitudes of a matrix z to range [0,1]
function z = normalize(z)
minz = min(min(abs(z)));
maxz = max(max(abs(z)));
z = (z- minz)/(maxz-minz);

%%

%interp_colormap
% Function which interpolates current colourmap to yield better graduated
% shading. N is number of possible colours.
function interp_colormap( N )

%Get current colourmap
map = colormap;

%Initialise new colormap
new_map = ones(N,3);

%Get size of current colormap and initalise red,green,blue vectors
dim = size(map);
R = ones(1,dim(1));
G = ones(1,dim(1));
B = ones(1,dim(1));
RR = ones(1,N);
GG = ones(1,N);
BB = ones(1,N);

%Populate these with current colormap
R(:) = map(:,1);
G(:) = map(:,2);
B(:) = map(:,3);

%Interpolate to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x );
GG = interp1( 1:dim(1), G, x );
BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:);
new_map(:,2) = GG(:);
new_map(:,3) = BB(:);

%Set colormap to be new map
colormap( new_map );

%End of code