%circle_intersect
% This program computes the intersection of two circles, whose radii and centres
% are specified.
%
% LAST UPDATED by Andy French May 2016

function circle_intersect

%% Inputs %%

%Circle #1

%x and y coordinates of centre
a1 = 1;
b1 = -1;

%Radius
r1 = 3;

%Circle #2

%x and y coordinates of centre
a2 = -4;
b2 = 4;

%Radius
r2 = 6;

%Fontsize
fsize = 14;

%%

%Compute intersections (imaginary solutions will be given as NaN)

%Coefficients of linear equation y = mx + c found from finding the
%difference between the expanded forms of the two circle equations
if (b2==b1) && (a1 ~=a2)
    x = (r1^2 - r2^2)/(2*(a2-a1));
    xplus = x;
    xminus = x;
    yminus = b1 - sqrt( r1^2 - (x-a1)^2 );
    yplus = b1 + sqrt( r1^2 - (x-a1)^2 );
    
elseif (b2==b1) && (a1 ==a2)
    xplus = NaN;
    xminus = NaN;
    yplus = NaN;
    yminus = NaN;
else
    m = (a1-a2)/(b2-b1);
    c = (r1^2 - r2^2 - a1^2 - b1^2 + a2^2 + b2^2)/(2*(b2-b1));
    
    %Coefficients of quadratic in x vx^2 + ux + w = 0 found by back substituting y = mx + c into
    %one of the circle equations (x-a1)^2 + (y-b1)^2 = r1^2
    v = 1 + m^2;
    u = -2*a1 + 2*m*c -2*b1*m;
    w = c^2 -2*b1*c - r1^2 + a1^2 + b1^2;
    
    %Solve quadratic to get x intersection coordinates of the circles
    xplus = (-u + sqrt( u^2 - 4*v*w ))/(2*v);
    xminus = (-u - sqrt( u^2 - 4*v*w ))/(2*v);
    
    %Obtain corresponding y values
    yplus = m*xplus + c;
    yminus = m*xminus + c;
    
    %Set NaN results if imaginary solutions (i.e. no intersections)
    if u^2 - 4*v*w < 0
        xplus = NaN;
        xminus = NaN;
        yplus = NaN;
        yminus = NaN;
    end 
end

%Plot circles
N = 100;
t = linspace(0,2*pi,N);
x1 = a1 + r1*cos(t);
y1 = b1 + r1*sin(t);
x2 = a2 + r2*cos(t);
y2 = b2 + r2*sin(t);
figure('name','circle intersect','color',[1 1 1]);
plot(x1,y1,'b')
hold on
plot(x2,y2,'r')
plot(xminus,yminus,'m*');
plot(xplus,yplus,'g*');
axis equal
legend({
    ['(x ',signstr(a1),')^2 + (y ',signstr(b1),')^2 = ',num2str(r1^2)],...
    ['(x ',signstr(a2),')^2 + (y ',signstr(b2),')^2 = ',num2str(r2^2)],...
    ['First intersection = (',num2str(xminus),',',num2str(yminus),')'],...
    ['Second intersection = (',num2str(xplus),',',num2str(yplus),')']
    },'location','eastoutside')
plot(a1,b1,'bx',a2,b2,'rx')
grid on
set(gca,'fontsize',fsize)
xlabel('x','fontsize',fsize)
ylabel('y','fontsize',fsize)
title('Intersection of circles','fontsize',fsize)

%Set axes limits and draw x,y axes
xlims = get(gca,'xlim');
ylims = get(gca,'ylim');
xmin = xlims(1);
xmax = xlims(2);
ymin = ylims(1);
ymax = ylims(2);
xmin = min([0,xmin]);
xmax = max([0,xmax]);
ymin = min([0,ymin]);
ymax = max([0,ymax]);
plot([0,0],[ymin,ymax],'k','linewidth',1)
plot([xmin,xmax],[0,0],'k','linewidth',1)
xlim([xmin,xmax])
ylim([ymin,ymax])

%Save PNG output
print(gcf,['circle intersect ',strrep(datestr(now),':','-')],'-dpng','-r300');
close(gcf);

%%

%Function for creation of signed strings for use in brackets e.g. ( x + 3 )
function s = signstr(x)
if x<0
    s = ['+ ',num2str(abs(x))];
else
    s = ['- ',num2str(abs(x))];
end

%End of code


