%cycloid
% Function which draws cycloids - the family of curves which are the locus of
% the tip of a radial spoke of a circle which is rolling over another,
% fixed, circle without slipping.
%
% LAST UPDATED by Andy French February 2014
%
% CONTROLS:
%
% q            Quit
% p            Print screen
% r            Reset
% w            Wait (i.e. pause animation)
% uparrow      Increase rolling circle radius
% downarrow    Decrease rolling circle radius
% leftarrow    Decrease radial spoke distance
% rightarrow   Incresae radial spoke distance
% t            Toggle trail
% f1           Decrease number of loops to form cycloid
% f2           Increase number of loops to form cycloid
% m            Start writing indexed frames to a directory


function cycloid
global d

%% Default parameters %%

%Radius of fixed circle
R_def = 3;

%Radius of rolling circle
r_def  = 1;

%Radial spoke distance from the rolling
%circle centre
a_def  = r_def;

%Initial polar angle of touching point of fixed and rolling circles /radians
theta = pi/2;

%Initially show cycloid
d.show_cycloid = 1;

%Loops of cycloid
M = 1;

%%

%Fixed parameters

%Change in fixed circle angle per iteration /radians
dtheta = 0.01;

%Change in number of loops
dM = 1;

%Change in a
da = 0.1;

%Change in r
dr = 0.2;

%Number of points for entire cycloid per loop
N = 1000;

%Number of data points for circles
Nc = 1000;

%Fontsize for title
fsize = 18;

%Linewidth of cycloid
lwidth = 1;

%Line colour of cycloid
cycloid_RGB = [1,0,0];

%Line colour of fixed circle
fcircle_RGB = [0,0,0];

%Line colour of rolling circle
rcircle_RGB = [0,0,1];

%%

%Initialize other elements of the data structure d
d.quit = 0;
d.print = 0;
d.movie = 0;
d.reset = 0;
d.wait = 0;
d.update_cycloid = 0;
d.trail = 0;
d.R = R_def;
d.r = r_def;
d.dr = dr;
d.a = a_def;
d.da = da;
d.M = M;
d.dM = dM;
d.frame = 0;

%Create default figure
fig = figure('name','cycloid','color',[1 1 1],'KeyPressFcn',@keypressfunc,'renderer','painters');

[xf,yf,xr,yr,xc,yc,pr,pf,prc,pc,p,pp] = create_default_figure( d.R, d.r, d.a, N,d.M, Nc, theta,...
    d.show_cycloid, fsize, lwidth, cycloid_RGB, fcircle_RGB, rcircle_RGB, fig );

%

%Animation loop
while d.quit == 0
    
    %Check if reset key 'r' has been pressed. If so return to defaults
    if d.reset == 1
        cla;
        d.reset = 0;
        [xf,yf,xr,yr,xc,yc,pr,pf,prc,pc,p,pp] = create_default_figure( d.R, d.r, d.a, N,d.M, Nc, theta,...
            d.show_cycloid, fsize, lwidth, cycloid_RGB, fcircle_RGB, rcircle_RGB, fig );
        xlimits = get(gca,'xlim');
        ylimits = get(gca,'ylim');
    end
    
    %Update frame if wait key (w) is turned off
    if d.wait == 0;
        
        %Update fixed circle polar angle
        theta = theta + dtheta;
        
        %Update coordinates of rolling circle
        [x,y,xr,yr] = update_rolling_circle( d.R, d.r, d.a, theta, Nc );
        set( p, 'xdata', [(d.R+d.r)*cos(theta), x], 'ydata', [(d.R+d.r)*sin(theta),y]);
        set( prc, 'xdata', (d.R+d.r)*cos(theta), 'ydata', (d.R+d.r)*sin(theta) );
        
        %Update current cycloid position
        set( pp, 'xdata', x, 'ydata', y );
        if d.trail == 1
            plot(x,y,'r.','markersize',0.1);
        end
        set( pr, 'xdata', xr, 'ydata', yr);
        
        %Toggle entire cycloid trace, if 'c' key is pressed
        if d.show_cycloid == 1
            set(pc,'visible','on');
            if d.update_cycloid == 1;
                [xc,yc,L] = create_cycloid( d.R, d.r, d.a, N, theta,d.M );
                set( gca, 'xlim',[-L,L], 'ylim',[-L,L] );
                d.update_cycloid =0;
            end
            set(pc,'xdata', xc, 'ydata', yc);
        else
            set(pc,'visible','off');
        end
    end
    
    %Update cycloid filename
    [fname, ax_title] = cycloid_title( d.R, d.r, d.a, theta,d.M );
    title(ax_title);
    
    %Check if print key 'p' has been pressed. If so, print screen to a PNG
    %file
    if d.print==1
        print(fig,[fname,'.png'],'-dpng','-r300');
        d.print = 0;
    end
    
    %Create movie frames (JPEG)
    if d.movie==1
        d.frame = d.frame + 1;
        print(fig,[d.movie_dir,'\frame',leadingzero(d.frame,5),'.jpg'],'-djpeg','-r300');
    end
    
    %Update figure
    drawnow;
    pause(0.01);
end
close(fig);

%%

%Create 'entire' cycloid coordeinates
function [xc,yc,L] = create_cycloid( R, r, a, N, theta, M )
t = theta + linspace(0,2*pi*M,N*M);
xc = (R+r)*cos(t) - a*cos( (R+r)*t/r );
yc = (R+r)*sin(t) - a*sin( (R+r)*t/r );
L1 = abs(R) + 2*abs(r) ;
L2 = abs(R) + abs(r) + abs(a);
L3 = 2*abs(r);
L = max([L1,L2,L3]);

%%

%Create circles fixed (f) and rolling (r)
function [xf,yf,xr,yr] = create_circles( R, r, theta, Nc )
t = linspace(0,2*pi,Nc);
xf = R*cos(t);
yf = R*sin(t);
xr = (R+r)*cos(theta) + r*cos(t);
yr = (R+r)*sin(theta) + r*sin(t);

%%

%Update rolling circle and tip of radial spoke coordinate
function [x,y,xr,yr] = update_rolling_circle( R, r, a, theta, Nc )
t = linspace(0,2*pi,Nc);
xr = (R+r)*cos(theta) + r*cos(t);
yr = (R+r)*sin(theta) + r*sin(t);
x = (R+r)*cos(theta) - a*cos( (R+r)*theta/r );
y = (R+r)*sin(theta) - a*sin( (R+r)*theta/r );

%%

%Generate text string for cycloid filename and axes title
function [fname, ax_title] = cycloid_title( R, r, a, theta, M )
ax_title = ['Cycloid: R=',num2str(R),', r=',num2str(r),', a=',num2str(a),', theta=',num2str(theta),', M=',num2str(M) ];
fname = strrep( ax_title,'.','p' );
fname = strrep( ax_title,':',' ' );

%%

%Create default figure
function [xf,yf,xr,yr,xc,yc,pr,pf,prc,pc,p,pp] = create_default_figure( R, r, a, N,M, Nc, theta, show_cycloid, fsize, lwidth, cycloid_RGB, fcircle_RGB, rcircle_RGB,fig )

%Generate circles and cycloid
[xf,yf,xr,yr] = create_circles( R, r, theta, Nc );
[xc,yc,L] = create_cycloid( R, r, a, N, theta,M );

%Update figure
pf = plot(xf,yf, 'color', fcircle_RGB, 'linewidth', lwidth );
hold on;
pr = plot(xr,yr, 'color', rcircle_RGB, 'linewidth', lwidth );
p = plot( [(R+r)*cos(theta), xc(1)] ,[(R+r)*sin(theta),yc(1)],'r-');
pp = plot( xc(1),yc(1),'r*');
pc = plot(xc,yc, 'color', cycloid_RGB, 'linewidth', lwidth );
prc = plot( (R+r)*cos(theta), (R+r)*sin(theta), 'b*' );
axis equal;
axis manual
axis off;

%Set limits to enclose the entire cycloid
set( gca, 'xlim',[-L,L], 'ylim',[-L,L] );

%%

%Zeros prior to a number
function s = leadingzero(x,N)
s = [strrep(blanks(N-1 - fix(log10(x))),' ','0'),...
    num2str(x,['%',num2str(N),'.0f'])];

%%

%Key press handler
function keypressfunc( fig,evnt )

%Allow read-write usage of global variable d
global d

%Get the string of the current key press
s = get( fig,'currentkey');

%Actions, depending on the key pressed...

if strcmp(s,'q') == 1
    %Quit
    d.quit = 1;
elseif strcmp(s,'p') == 1
    %Print screen
    d.print = 1;
elseif strcmp(s,'r') == 1
    %Reset cycloid animation
    d.reset = 1;
elseif strcmp(s,'w') == 1
    %Pause animation
    if d.wait ==1
        d.wait = 0;
    else
        d.wait = 1;
    end
elseif strcmp(s,'c') == 1
    %Toggle complete cycloid
    if d.show_cycloid==1
        d.show_cycloid =0;
    else
        d.show_cycloid =1;
    end
elseif strcmp(s,'uparrow') == 1
    %Increase rolling circle radius
    d.r = d.r + d.dr;
    d.update_cycloid = 1;
elseif strcmp(s,'downarrow') == 1
    %Decrease rolling circle radius
    d.r = d.r - d.dr;
    d.update_cycloid = 1;
elseif strcmp(s,'leftarrow') == 1
    %Decrease size of radial spoke from the rolling circle
    d.a = d.a + d.da;
    d.update_cycloid = 1;
elseif strcmp(s,'rightarrow') == 1
    %Increase size of radial spoke from the rolling circle
    d.a = d.a - d.da;
    d.update_cycloid = 1;
elseif strcmp(s,'m') == 1
    %Toggle writing of movie frames
    if d.movie == 1
        d.movie = 0;
    else
        d.movie = 1;
        %Create movie directory
        d.movie_dir = strrep( datestr(now),':','-' );
        mkdir( pwd, d.movie_dir );
        d.frame = 0;
    end
elseif strcmp(s,'t') == 1
    %Toggle trail
    if d.trail == 1
        d.trail = 0;
    else
        d.trail = 1;
    end
elseif strcmp(s,'f1') == 1
    %Add loops to 'entire' cycloid
    d.M = d.M - d.dM;
    d.update_cycloid = 1;
elseif strcmp(s,'f2') == 1
    %Reduce 'entire' cycloid loops
    d.M = d.M + d.dM;
    d.update_cycloid = 1;
end

%End of code