%fractaldim
% Demonstrates the computation of the fractal dimension of a random
% collection of x,y,z points.
%
% LAST UPDATED by Andy French May 2016

% Syntax of FRACDIM_DEMO.m called by this script
%
% FRACDIM_DEMO(DEMO_TYPE,NUM_POINTS,ITERATIONS,METHOD)
%
% NUM_POINTS is the number of data points computed
% ITERATIONS is the number of iterations performed by FRACDIM
% METHOD is a string i.e. either 'boxcount' or 'divstep'
% DEMO_TYPE is a string from the following list:
% 'normxy'   normal distributed x,y variables.
% 'normxyz'  normal distributed x,y,z variables.
% 'ellipse'  points are samples along an elliptical curve.
% 'line'     points are sampled along a straight line.
% 
% Output is a series of data plots showing boxes for each iteration.
% This is not shown for DEMO_TYPE='normxyz', as transparent cubes would have
% to be drawn. In addition, a graph of log(1/box size) vs log(box count) is
% shown for METHOD='boxcount' and a graph of log(divider length) vs log(path length) is
% shown for METHOD='divstep'.

function fractaldim

fracdim_demo('normxy',100,15,'boxcount')
%fracdim_demo('normxyz',100,15,'boxcount')
%fracdim_demo('ellipse',100,15,'boxcount')
%fracdim_demo('line',100,15,'boxcount')

%%

%FRACDIM_DEMO Demonstration of Fractal Dimension 
% calculator 'FRACDIM'
%
% Andy French. September 2003.
%
% Syntax:
%
% FRACDIM_DEMO(DEMO_TYPE,NUM_POINTS,ITERATIONS,METHOD)
%
% NUM_POINTS is the number of data points computed
% ITERATIONS is the number of iterations performed by FRACDIM
% METHOD is a string i.e. either 'boxcount' or 'divstep'
% DEMO_TYPE is a string from the following list:
% 'normxy'   normal distributed x,y variables.
% 'normxyz'  normal distributed x,y,z variables.
% 'ellipse'  points are samples along an elliptical curve.
% 'line'     points are sampled along a straight line.
% 
% Output is a series of data plots showing boxes for each iteration.
% This is not shown for DEMO_TYPE='normxyz', as transparent cubes would have
% to be drawn. In addition, a graph of log(1/box size) vs log(box count) is
% shown for METHOD='boxcount' and a graph of log(divider length) vs log(path length) is
% shown for METHOD='divstep'.
%
% Examples:
% fracdim_demo('normxy',100,15,'boxcount')
% fracdim_demo('normxyz',100,15,'boxcount')
% fracdim_demo('ellipse',100,15,'boxcount')
% fracdim_demo('line',100,15,'boxcount')

function fracdim_demo(demo_type,num_points,iterations,method)

%Set destination directory of plot images
set_directory=[];

if strcmp(method,'boxcount')==1
    if strcmp(demo_type,'normxyz')==1
        
        %Compute x and y vectors and POINTS array
        x=normrnd(0,3,1,num_points); y=normrnd(0,3,1,num_points);
        z=normrnd(0,3,1,num_points);
        for n=1:num_points 
            points(n,:)=[x(n),y(n),z(n)];
        end
        
        %Run FRACDIM function
        output=fracdim(points,iterations,'boxcount',[],[]);
        fractal_dimension=output{1}
        error=output{2}
        a=output{3};
        count=output{4}; 
        
        %Plot log curves
        newfig=figure('color',[1 1 1]);
        plot(log(1./a),log(count),'b+');
        hold on;
        count_best_fit=output{1}*log(1./a)+log(output{5})*ones(1,iterations);
        plot(log(1./a),count_best_fit,'r-')
        xlabel('log(1/a)'); 
        ylabel('log(count)');
        title(['Fractal dimension=',num2str(fractal_dimension),' (+/-) ',num2str(error)]);
        
        %Save png file of this first plot
        filename=[set_directory,'graphs.png'];
        print( gcf,filename,'-dpng','-r300'); 
           
        %Plot data and centre of mass
        newfig=figure('color',[1 1 1]); 
        plot3(x,y,z,'b+');
        xlabel('x'); ylabel('y'); zlabel('z');
        hold on;
        
        %Plot centre of mass
        COM=(1/num_points)*sum([x.',y.',z.']); 
        plot3(COM(1),COM(2),COM(3),'r+'); 	
        
        %Save BMP file of this first plot
        filename=[set_directory,'data.png'];
        print( gcf,filename,'-dpng','-r300'); 
        
    else
        if strcmp(demo_type,'normxy')==1
            %Compute x and y vectors and POINTS array
            x=randnum('normal',0,3,1,num_points); y=randnum('normal',0,3,1,num_points);
        elseif strcmp(demo_type,'line')==1
            %Compute x and y vectors and POINTS array
            x=randnum('uniform'-10,10,1,100); y=2*x-0.5*ones(1,100);
        elseif strcmp(demo_type,'ellipse')==1
            %Compute x and y vectors and POINTS array
            test_ellipse=ellipse(100,10,6,pi/4,[3,5]);
            xx=test_ellipse{1}(:,1); yy=test_ellipse{1}(:,2);
            x=xx.'; y=yy.';
        else
            error('demo_type not recognised')    
        end
        
        %Run FRACDIM function
        for n=1:num_points 
            points(n,:)=[x(n),y(n)];
        end
        output=fracdim(points,iterations,'boxcount',[],[]);
        fractal_dimension=output{1}
        error=output{2}
        a=output{3};
        count=output{4};
        
        %Plot log curves
        newfig=figure('color',[1 1 1]);
        plot(log(a(1)./a),log(count),'b+');
        hold on;
        count_best_fit=output{1}*log(a(1)./a)+log(output{5})*ones(1,iterations);
        plot(log(a(1)./a),count_best_fit,'r-')
        xlabel('log(a_{max}/a)'); 
        ylabel('log(count)');
        title(['Fractal dimension=',num2str(fractal_dimension),' (+/-) ',num2str(error)]);
        
        %Save a .png file of this first plot
        filename=[set_directory,'graphs.png'];
        print( gcf,filename,'-dpng','-r300'); 
        
        %Plot data and boxes
        for j=1:iterations
            close(newfig);
            newfig=figure('color',[1 1 1]);
            axes('Nextplot','add'); 
            [A,B]=size(output{7}{j});
            
            %Plot boxes
            for i=1:A; 
                box=rect(a(j),a(j),0,output{7}{j}(i,:));
                patch(box(:,1),box(:,2),'g'); 
            end; 
            
            %Plot data
            plot(x,y,'b+'); 
            
            %Plot centre of mass
            COM=(1/num_points)*sum([x.',y.']); 
            plot(COM(1),COM(2),'r+'); 
            axis equal
            xlim([COM(1)-0.6*a(1),COM(1)+0.6*a(1)])
            ylim([COM(2)-0.6*a(1),COM(2)+0.6*a(1)])
            xlabel('x'); ylabel('y');
            grid on;
            
            %Save .png file of this first plot
            filename=[set_directory,leadingzero(j,3),'.png'];
            print( gcf,filename,'-dpng','-r300'); 
        end
    end
    close(gcf);
else
    error('Alternative method as yet unsupported')    
end

%%

%FRACDIM Function that estimates the fractal dimension of a set
% of M points defined by N component position vectors.
% 
% Andy French. September 2003.
%
% Syntax:
%
% OUTPUT=FRACDIM(POINTS,ITERATIONS,METHOD,MAX_BOX_SIZE,BOX_CENTER)
%
% POINTS is an MxN matrix of row vectors describing the set of points
% defined in an N-dimensional Cartesian space.
% ITERATIONS is the number of iterations used to compute the result. The
% higher this number, the more accurate the result will be, but at a cost
% of greater computational time.
% METHOD chooses method of fractal dimension computation.
% 'boxcount' uses the box counting method.
% 'divstep' uses the divider-stepper method.
% MAX_BOX_SIZE is the maximum box size used for the first iteration,
% (or the maximum divider length if 'divstep' is used).
% If MAX_BOX_SIZE=[] then FRACDIM will compute the maximum box size from
% the data. (Twice the maximum radius between data centre of mass and data
% points).
% BOX_CENTER is the center of the initial box. (1xN array).
% If BOX_CENTER=[] then FRACDIM uses the data COM as the center.
% OUTPUT is a structured array.
% OUTPUT{1} is the mean fractal dimension
% OUTPUT{2} is the error in this measurement
% OUTPUT{3} is a vector of length ITERATIONS of box sizes (if
% METHOD='boxcount') or divider length if 'divstep' is used.
% OUTPUT{4} is a vector of length IERATIONS giving the length of a path
% passing through all the points (in order) measured with 'dividers' of
% length corresonding to the element in OUTPUT{3}, if METHOD=='divstep'
% or the number of boxes that contain points if METHOD=='boxcount'
% 
% If METHOD=='divstep'
% The fractal dimension D is given by the solution to the equation
% L(R)=C*R^(1-D) where R is the divider 'radius,' C is a constant and
% L is the length of the point path as measured by dividers of length R.
%
% If METHOD=='boxcount'
% The fractal dimension D is the solution to the equation
% B(A)=(A0/A)^D where B is the number of boxes that contain points and A is
% the box size. Box volume is A^N and A0 is the size of the initial box.
% A<=A0.
%
% OUTPUT{5} is the constant C
% OUTPUT{6} is the error in constant C
% OUTPUT{7} is a structured array (length ITERATIONS) of the box centres
% that contain points. i.e. the number of vectors in OUTPUT{7}{i}
% =OUTPUT{4}.
%
% Examples:
% fracdim_demo('normxy',100,15,'boxcount')
% fracdim_demo('normxyz',100,15,'boxcount')
% fracdim_demo('ellipse',100,15,'boxcount')
% fracdim_demo('line',100,15,'boxcount')

function output=fracdim(points,iterations,method,max_box_size,box_center)

if strcmp(method,'boxcount')==1
    
    %%%%%BOX COUNTING METHOD%%%%%
    
    %Find N and M
    [M,N]=size(points);
    
    if length(box_center)==0
        %Compute centre of mass of data
        COM=sum(points)/M;
    else
        COM=box_center;
    end
   
    if length(max_box_size)==0
        
        %Find vector of distances of all points to COM
        for m=1:M
            dist2COM(m)=vectorlength(points(m,:)-COM);
        end
        
        %Define largest 'N-box' size that will enncompass all the data points
        a_max=2*max(dist2COM);
        
    else
        a_max=max_box_size;
    end
    
    %Transform data points such that the origin corresponds with the bottom
    %edge of the box.
    for m=1:M
        points(m,:)=points(m,:)-COM+0.5*a_max*ones(1,N);    
    end
    
    %The number of boxes which encompass the data is obviously 1.
    %Hence initialise variable COUNT and box size A
    count(1)=1;
    a(1)=a_max;
    
    %Define structured array of box centers of boxes that contain points.
    boxcenters{1}=COM;
    
    %Now for the next i=2:ITERATIONS steps reduce box size to a_max/i
    %and check the i^N boxes within the volume of the original box for
    %points. If boxes contain points, add to boxcount(i) and remove points
    %from a list.
    
    for i=2:iterations
        
        %Define box size
        a(i)=a_max/i;
        
        %Compute matrix of box indices.
        %This will be an i^N x N matrix of permutations
        %of N integers varying from 1 to i.
        %
        %i.e. for an N=2 system of integers 1,2,3
        %possible combinations (box indices) are
        %
        % [1 1] [1 2] [1 3] [2 1] [2 2] [2 3] [3 1] [3 2] [3 3]
        %
        %i.e. 3^2 = 9 combinations of 2 integer pairs.
        box_indices=intperm(i,N);
        
        %Step through all i^N elements of BOX_INDICES
        %and compute boundaries of box.
        
        %For each box, find the points that belong in it. Count these boxes
        %and delete the points.
        
        %Copy array POINTS
        points_copy=points;
        
        %Initialize array BOXCOUNT
        boxcount=0;
        
        %Initialise array BOX_CENTER - this will be used to display the
        %filled boxed in demonstrations;
        box_center=[];
        
        for k=1:i^N
            for n=1:N
                box_coord_min(n)=(box_indices(k,n)-1)*a(i);
                box_coord_max(n)=box_indices(k,n)*a(i);
            end
            
            %Step though POINTS_COPY and check whether any points lie
            %within box. Do this by subtracting BOX_COORD_MIN from
            %POINTS(j,:) and checking whether all elements are positive.
            %Also subtract POINTS(j,:) from BOX_COORD_MAX and check whether
            %all these elements are positive. If both these tests are
            %successful then a point is in the box.
            
            [Mc,N2]=size(points_copy);
            
            %Initialise POINTCOUNT and INBOX_POINTNUM
            pointcount=0;
            inbox_pointnum=zeros(1,Mc);
            
            for j=1:Mc
                diff_min=points_copy(j,:)-box_coord_min;
                diff_max=-points_copy(j,:)+box_coord_max;
                min_check=sum(sign(diff_min));
                max_check=sum(sign(diff_max));
                if (min_check==N) & (max_check==N)
                    %Point in the box
                    pointcount=pointcount+1;
                    inbox_pointnum(j)=j;
                else
                    %Point not in box
                    inbox_pointnum(j)=0;
                end
            end
            
            %If POINTCOUNT>0, delete plots that fit in the box and add one
            %to BOXCOUNT
            
            if pointcount>0
                boxcount=boxcount+1;
                newpoints=[];
                for j=1:Mc
                    if inbox_pointnum(j)==0
                        newpoints=[newpoints;points_copy(j,:)];
                    end
                end
                
                %Compute box center and add to store - remebering to
                %transform back to the original frame of reference of the
                %points.
                box_center=[box_center;a(i)*(box_indices(k,:)-0.5*ones(1,N))+COM-0.5*a_max*ones(1,N)];
                
                %Update POINTS_COPY
                points_copy=newpoints;
                
            end      
            
        end
        
        %Store BOXCOUNT in array COUNT
        count(i)=boxcount;
        
        %Store BOX_CENTRE matrix in BOXCENTRES structured array
        boxcenters{i}=box_center;
        
    end
    
    %Compute fractal dimension and error by fitting a straight line
    %to a graph of ln(a_max/a) vs ln(count). Note constant C should be unity.
    
    [intercept,intercept_error,gradient,gradient_error]=linfitter(log(a_max./a),log(count));
    
    output{1}=gradient;                     %Fractal dimension
    output{2}=gradient_error;               %Error in fractal dimension
    output{3}=a;                            %Box size a
    output{4}=count;                        %Number of boxes that contain points
    output{5}=exp(intercept);               %Constant C
    output{6}=output{5}*gradient_error;     %Error in C
    output{7}=boxcenters;                   %Structured array of box centers for boxes that 
                                            %contain points.                                          
                                            
%%%%%Under construction........................                                            
                                                                                       
elseif method=='divstep'
    
    error('Method under development')
    %%%%%DIVIDER-STEPPER METHOD%%%%%
    
    %Find N and M
    [M,N]=size(points);
    
    %Compute vector of point separation distances
    for m=2:M
        separation(m-1)=vectorlength(points(m,:)-points(m-1,:));
    end
    clear m;
    
    %Find minimum and maximum separations - use this information
    %to construct a linearly space vector of divider lengths R.
    max_separation=max(separation);
    min_separation=min(separation);
    R=linspace(min_separation,max_separation,num_dividers);
    
    %Compute path length for each length divider
    for k=1:num_dividers
        
        %Initialise countable quantities
        divcount=0;     %Divider counter
        
        %Initialise start of 'divider path measurement'
        pathpoint=points(1,:);
        
        %Initialse matrix POINTSREQ.
        %This is the subset of POINTS used by the algorithm.
        %As the dividers step along the path one will not
        %require points already traversed.
        pointsreq=points;
        
        %Initialise binary variable STOP. If divider moves off end
        %of path, cease algorithm and return current DIVCOUNT.
        %This corresponds to STOP==1;
        stop=0;
        
        %Using the divider as a general (N-1)-Sphere generator,
        %compute intersections of sphere with path segments by firstly
        %finding all segments that have (i) one vertex vector of length
        %<=divider length and one >= divider length (all lengths measured from current PATHPOINT)
        %or (ii) both vertex vectors
        %have length > divider length and have shortest distance from current
        %PATHPOINT to line segment being less than divider length.
        
        while stop==0;
            
            %Shift origin to current pathpoint
            [M_req,N]=size(pointsreq);
            for m=1:M_req
                pointsreq(m,:)=pointsreq(m,:)-pathpoint;
            end
            
            clear m;
            
            %Initialise INTERSECTION_CHECK
            intersection_check=zeros(1,M_req-1);
            
            %Find intersections of line segments with (N-1)-sphere
            for m=2:M_req
                INTersections=lineintersect(R(k),pointsreq(m-1,:),pointsreq(m,:));
                intersection_vector=INTersections{1};
                intersection_check(m-1)=length(intersection_vector);
                intersections{m-1}=INTersections{1}; 
            end
            
            clear m;
            
            %Look though array INTERSECTION_CHECK and find first element that
            %==1.
            good_checks=find(intersection_check==N);
            
            %New pathpoint is the GOOD_CHECKS(1) element of INTERSECTIONS
            %If GOOD_CHECKS==[] then set STOP to 1. 
            if length(good_checks)==0
                stop=1;
            else
                
                pathpoint=intersections{good_checks(1)};
                clear intersections;
                
                %Update DIVCOUNT
                divcount=divcount+1;
                
                %Remove unrequired points from POINTREQ
                newpoints=[];
                for m=1:M_req
                    if m>good_checks(1)
                        newpoints=[newpoints;pointsreq(m,:)];
                    end
                end
                pointsreq=newpoints;
                
            end
            
        end
        
        %Compute measured pathlength L
        L(k)=R(k)*divcount;
        
    end
    
    %Compute fractal dimension and error by fitting a straight line
    %to a graph of ln(R) vs ln(L)
    
    [intercept,intercept_error,gradient,gradient_error]=linfitter(log(R),log(L));
    
    output{1}=1-gradient;                   %Fractal dimension
    output{2}=gradient_error;               %Error in fractal dimension
    output{3}=R;                            %Divider sizes R
    output{4}=L;                            %Length L of path measured by dividers
    output{5}=exp(intercept);               %Constant C
    output{6}=output{5}*gradient_error;     %Error in C
else
    disp('Invalid METHOD argument')
end

%%

%ELLIPSE Function to create an array of co-ordinates
% that can be used to plot an ellipse of arbitary orientation.
% The vertices are ordered in a clockwise fashion.
%
% Andy French. June 2003.
%
% Syntax:
%
% OUTPUT=ELLIPSE(N,SEMI_MAJ,SEMI_MIN,TILT,CENTRE)
%
% N is the number of points, minimum of 8
% SEMI_MAJ and SEMI_MIN are the lengths 
% and heights respectively of the ellipse.
% Note SEMI_MAJ must be >= SEMI_MIN
% TILT is the clockwise angle of ellipse tilt (in radians) of the
% SEMI_MAJ axis from the horizontal (x axis).
% CENTRE is the [x,y] vector specifying the centre of the ellipse.
%
% OUTPUT{1} is an Nx2 array of ellipse [x,y] vertices
% OUTPUT{2} are the co-ordinates of the -,+ foci of the ellipse 
% This is a 2x2 array with the - foci being the first row.
% OUTPUT{3} is the ellipse eccentricity
%
% Example: test_ellipse=ellipse(100,10,6,pi/4,[3,5]);
% plot([test_ellipse{1}(:,1);test_ellipse{1}(1,1)],[test_ellipse{1}(:,2);test_ellipse{1}(1,2)]);
% hold on; axis equal;
% plot(test_ellipse{2}(:,1),test_ellipse{2}(:,2),'r+')

%%

%vectorlength
function Xout=vectorlength(Xin)
Xout=sqrt(sum(Xin.*Xin));

%%

%COSINE finds the cosine of the INPUT,
% but actually gives zero when the input is odd integer*PI/2
% Amazingly MATLAB doesn't!!!! 
%
% Andy French. June 2003.
% 
% Syntax: OUTPUT=COSINE(X)

function output=cosine(x)

for n=1:length(x)
    if isodd(2*x(n)/pi)
        output(n)=0;
    else
        output(n)=cos(x(n));
    end
end

%%

%SINE finds the sine of the INPUT,
% but actually gives zero when the input is PI*integer.
% Amazingly MATLAB doesn't!!!! 
%
% Andy French. June 2003.
% 
% Syntax: OUTPUT=SINE(X)

function output=sine(x)

for n=1:length(x)
    if isinteger(x(n)/pi)==1
        output(n)=0;
    else
        output(n)=sin(x(n));
    end
end

%%

%RECT Function to create an array of co-ordinates
% that can be used to plot a rectangle of arbitary orientation.
% Vertices will be ordered in a clockwise fashion.
%
% Andy French. June 2003.
%
% Syntax:
%
% OUTPUT=RECT(LENGTH,WIDTH,TILT,CENTRE)
%
% LENGTH and WIDTH are the edge lengths and widths of the rectangle.
% Note LENGTH must be >= WIDTH
% TILT is the clockwise angle of rectangle tilt (in radians) of the
% LENGTH axis from the horizontal (x axis).
% CENTRE is the [x,y] vector specifying the centre of the rectangle.
%
% OUTPUT is an 4x2 array of rectangle [x,y] vertices
%
% Example: test_rec=rect(10,6,pi/4,[3,5]);
% plot([test_rec(:,1);test_rec(1,1)],[test_rec(:,2);test_rec(1,2)]); 
% hold on; axis equal; plot(3,5,'r+')

function output=rect(length,width,tilt,centre)

%Define rectangle in co-ordinates centred on rectangle centre
%and aligned with LENGTH and WIDTH axes

topright=[length/2;width/2];
topleft=[-length/2;width/2];
botright=[length/2;-width/2];
botleft=[-length/2;-width/2];

un_tilted_verts=[topleft,topright,botright,botleft];

for n=1:4
   
    %Rotate elements by tilt
    xy_vec=[cosine(tilt),sine(tilt);-sine(tilt),cosine(tilt)]*un_tilted_verts(:,n);
    
    %Shift centre of rectangle
    col_vec_array(:,n)=xy_vec+[centre(1);centre(2)];
end

%Define output
output=col_vec_array.' ;

%%

%RANDNUM random number generator.
%
% Andy French. November 2003.
%
% Syntax:   output=randnum(type,param1,param2,rows,columns)
%
% -type is a string which determines the random number distribution.
%  Currently type can be: 'weibull', 'normal' ,'poisson', 'uniform' 
%  'integer uniform' , 'rayleigh','binomial', 'gamma'  
% -The distribution is characterised by a maximum of two parameters.
%  param1 and param2. Single parameter distributions will only use the
%  first paramater.
% -The output shall be a matrix of random numbers of dimensions rows x
%  columns.
%
% Example:  output=randnum('binomial',5,3,4,4)
%
% Defenition of distributions:
% 
% BINOMIAL  output=randnum('binomial',n,p,rows,columns)   
%   Discrete distribution so random numbers are integers.
%   The probability that r out of n trials results in outcome A out of
%   possible outcomes A and B, given the probability of outcome A is p for
%   each (independant) trial** is:
%
%   P(r|n,p)=N(n,r)*( p^r )*( (1-p)^(n-r) )   where r,n are integers, 0<p<1.
%
%   N(n,r)=n!/((n-r)!r!)   Note in MATLAB n! is computed by  prod(1:n)
%   randnum picks a value r from the binomial distribution P(r|n,p).
%
%   **the probability of outcome B is thus 1-p for
%   each trial)
% 
%
%
% NORMAL    output=randnum('normal',mu,sigma,rows,columns) 
%
% WEIBULL   output=randnum('normal',a,b,rows,columns) 
%
% UNIFORM   output=randnum('normal',n,[],rows,columns) 
%
% INTERGER UNIFORM   output=randnum('normal',mu,sigma,rows,columns) 
%
% POISSON   output=randnum('normal',lambda,[],rows,columns) 
%
% GAMMA     output=randnum('normal',a,b,rows,columns) 
%
% RAYLEIGH  output=randnum('normal',b,[],rows,columns) 

function output=randnum(type,param1,param2,rows,columns)

%WEIBULL
if strcmp(type,'weibull')
    a=param1;
    b=param2;
    
    % Handle the simplest situation, all "nice" parameter values, efficiently
    k = find(a > 0 & b > 0);
    if (prod([size(a),size(b)]) == 1 & length(k) == 1) ... % scalar params
            | (length(k) == rows*columns)     % one or more matrix params
        r = (-log(rand(rows,columns))./a) .^ (1 ./ b);
        
        % Handle calls with some illegal parameter values
    else
        r = repmat(NaN, rows, columns);
        if length(k) > 0
            if prod(size(a)) > 1, a = a(k); end
            if prod(size(b)) > 1, b = b(k); end
            r(k) = (-log(rand(size(k)))./a) .^ (1 ./ b);
        end
    end 
    output=r;
    
%%    
    
%NORMAL    
elseif strcmp(type,'normal')
    mu=param1;
    sigma=param2;
    r = randn(rows,columns) .* sigma + mu;
    
    % Fill in elements corresponding to illegal parameter values
    if prod(size(sigma)) > 1
        r(sigma < 0) = NaN;
    elseif sigma < 0
        r(:) = NaN;
    end
    output=r;

%%

%POISSON    
elseif strcmp(type,'poisson')
    lambda=param1;
    if (prod(size(lambda)) == 1)
        lambda = lambda(ones(rows*columns,1));
    else
        lambda = lambda(:);
    end
    
    %Initialize r to zero.
    r = zeros(rows, columns);
    j = (1:(rows*columns))';   % indices remaining to generate
    
    % For large lambda, use the method of Ahrens and Dieter as
    % described in Knuth, Volume 2, 1998 edition.
    k = find(lambda >= 15);
    if ~isempty(k)
        alpha = 7/8;
        lk = lambda(k);
        m = floor(alpha * lk);
        
        % Generate m waiting times, all at once
        x = randnum('gamma',m,1,1,1);
        t = x <= lk;
        
        % If we did not overshoot, then the number of additional times
        % has a Poisson distribution with a smaller mean.
        r(k(t)) = m(t) + randnum('poisson',lk(t)-x(t),[],1,1);
        
        % If we did overshoot, then the times up to m-1 are uniformly
        % distributed on the interval to x, so the count of times less
        % than lambda has a binomial distribution.
        r(k(~t)) = randnum('binomial',m(~t)-1, lk(~t)./x(~t),1,1);
        j(k) = [];
    end

    % For small lambda, generate and count waiting times.
    p = zeros(length(j),1);
    while ~isempty(j)
        p = p - log(rand(length(j),1));
        kc = [1:length(k)]';
        t = (p < lambda(j));
        j = j(t);
        p = p(t);
        r(j) = r(j) + 1;
    end
    
    % Return NaN if LAMBDA is negative.
    r(lambda < 0) = NaN;
    output=r;

%%

%UNIFORM
elseif strcmp(type,'uniform')
    a=param1;
    b=param2;
    r = a + (b-a) .* rand(rows,columns);
    
    % Fill in elements corresponding to illegal parameter values
    if prod([size(a),size(b)]) > 1
        r(a > b) = NaN;
    elseif a > b
        r(:) = NaN;
    end
    output=r;
    
%%    

%INTEGER UNIFORM
elseif strcmp(type,'integer uniform')
    n=param1;
    r = ceil(n .* rand(rows,columns));
    
    % Fill in elements corresponding to illegal parameter values
    if prod(size(n)) > 1
        r(n < 0 | round(n) ~= n) = NaN;
    elseif n < 0 | round(n) ~= n
        r(:) = NaN;
    end
    output=r;
    
%%

%BINOMIAL
elseif strcmp(type,'binomial') 
    %"param1" is n "param2" is p
    n=param1;
    p=param2;
    
    % Handle the scalar params case efficiently
    if prod([size(p),size(n)]) == 1 % scalar params
        if (0 <= p & p <= 1) & (0 < n & round(n) == n)
            r = sum(rand(rows,columns,n) < p, 3);
        else
            r = repmat(NaN, rows, columns);
        end
        
        % Handle the scalar n case efficiently
    elseif prod(size(n)) == 1
        if 0 < n & round(n) == n
            r = sum(rand(rows,columns,n) < repmat(p, [1,1,n]), 3);
            r(p < 0 | 1 < p) = NaN;
        else
            r = repmat(NaN, rows, columns);
        end
        
        % Handle the non-scalar params case
    else 
        if prod(size(p)) == 1, p = repmat(p, rows, columns); end
        r = zeros(rows,columns);
        for i = 1:max(max(n))
            k = find(n >= i);
            r(k) = r(k) + (rand(size(k)) < p(k));
        end
        r(p < 0 | 1 < p | n < 0 | round(n) ~= n) = NaN;
    end
    output=r;
    
%%

%GAMMA
elseif strcmp(type,'gamma')
    a=param1;
    b=param2;
    
    % Initialize r to zero.
    lth = rows*columns;
    r = zeros(lth,1);
    a = a(:); b = b(:);
    
    if length(a) == 1, a = a*ones(lth,1); end
    if length(b) == 1, b = b*ones(lth,1); end
    
    % If a == 1, then gamma is exponential. (Devroye, page 405).
    k = find(a == 1);
    if any(k)
        r(k) = -b(k) .* log(rand(size(k)));
    end 
    
    k = find(a < 1 & a > 0);
    % (Devroye, page 418 Johnk's generator)
    if any(k)
        c = zeros(lth,1);
        d = zeros(lth,1);
        c(k) = 1 ./ a(k);
        d(k) = 1 ./ (1 - a(k));
        accept = k;
        while ~isempty(accept)
            u = rand(size(accept));
            v = rand(size(accept));
            x = u .^ c(accept);
            y = v .^ d(accept);
            k1 = find((x + y) <= 1); 
            if ~isempty(k1)
                e = -log(rand(size(k1))); 
                r(accept(k1)) = e .* x(k1) ./ (x(k1) + y(k1));
                accept(k1) = [];
            end
        end
        r(k) = r(k) .* b(k);
    end
    
    % Use a rejection method for a > 1.
    k = find(a > 1);
    % (Devroye, page 410 Best's algorithm)
    bb = zeros(size(a));
    c  = bb;
    if any(k)
        bb(k) = a(k) - 1;
        c(k) = 3 * a(k) - 3/4;
        accept = k; 
        while ~isempty(accept)
            m = length(accept);
            u = rand(m,1);
            v = rand(m,1);
            w = u .* (1 - u);
            y = sqrt(c(accept) ./ w) .* (u - 0.5);
            x = bb(accept) + y;
            k1 = find(x >= 0);
            if ~isempty(k1)
                z = 64 * (w .^ 3) .* (v .^ 2);
                k2 = (z(k1) <= (1 - 2 * (y(k1) .^2) ./ x(k1)));
                k3 = k1(k2);
                r(accept(k3)) = x(k3); 
                k4 = k1(~k2);
                k5 = k4(log(z(k4)) <= (2*(bb(accept(k4)).*log(x(k4)./bb(accept(k4)))-y(k4))));
                r(accept(k5)) = x(k5);
                omit = [k3; k5];
                accept(omit) = [];
            end
        end
        r(k) = r(k) .* b(k);
    end
    
    % Return NaN if a or b is not positive.
    r(b <= 0 | a <= 0) = NaN;
    r = reshape(r,rows,columns);
    output=r;

%%

%RAYLEIGH
elseif strcmp(type,'rayleigh')
    b=param1;
    r = sqrt(randn(rows,columns).^2 + randn(rows,columns).^2) .* b;
    % Fill in elements corresponding to illegal parameter values
    if prod(size(b)) > 1
        r(b < 0) = NaN;
    elseif b < 0
        r(:) = NaN;
    end
    output=r;
    
%%   

%Default to UNIFORM
else   
    a=param1;
    b=param1;
    r = a + (b-a) .* rand(rows,columns);
    
    % Fill in elements corresponding to illegal parameter values
    if prod([size(a),size(b)]) > 1
        r(a > b) = NaN;
    elseif a > b
        r(:) = NaN;
    end
    output=r;    
end

%%

%LINFITTER Function that computes intercept C , gradient M (and errors in these) of
% a straight line Y=MX+C fitted in a least-squares sence to vectors of X and Y
% data.
%
% Andy French. September 2003.
%
% Syntax:
%
% [C,C_ERROR,M,M_ERROR]=linfitter(X,Y)
%
% X and Y are vectors of the same length.
% The method is from p43 of "Practical Physics" by G.L. Squires
%
% Example x=unifrnd(0,1,1,100); m=normrnd(2,0.3,1,100); c=normrnd(1,0.3,1,100);
% y=m.*x+c; [C,c_error,M,m_error]=linfitter(x,y);
% Y=M*x+ones(1,100)*C; plot(x,y,'b+'); hold on; plot(x,Y,'r-')

function [c,c_error,m,m_error]=linfitter(x,y)

%Find number of data points
N=length(x);

%Compute intermediate parameters
x_bar=(1/N)*sum(x);
y_bar=(1/N)*sum(y);
D=sum(x.*x)-(1/N)*sum(x)*sum(x);
E=sum(x.*y)-(1/N)*sum(x)*sum(y);
F=sum(y.*y)-(1/N)*sum(y)*sum(y);

%Compute outputs
m=E/D;
c=y_bar-m*x_bar;
m_error=sqrt((D*F-E*E)/(D*D*(N-2)) );
c_error=m_error*sqrt(D/N + x_bar*x_bar);

%%

%LEADINGZERO
%
%Function to return an N character string consisting of the integer part 
%of argument X with leading zeros as necessary
%
%Syntax:
%
%outstring=leadingzero(integer array of indices (X) ,N)

function outstring = leadingzero(X,N)
outstring = [strrep(blanks(N-1 - fix(log10(X))),' ','0'),num2str(X,3)];

%%

%ISVECTOR Logical function that tests whether an array is a vector (one dimensional) or not.
% Andrew French. Cambridge University. March 2002.
%
% Syntax: OUTPUT=ISVECTOR(INPUT)
%
% OUTPUT=1 if INPUT is a vector
% OUTPUT=0 if INPUT is not a vector

function output=isvector(input)

if ischar(input)==0
    
    %Get array of dimensions of input
    dim_input=size(input) ;
    
    if length(dim_input)==2
        
        %Find smallest element of (NxM array)
        %If this is 1 then input is a vector.
        min_dim_length=min(dim_input)  ;
        
        if min_dim_length==1 
            output=1 ;  
        else 
            output=0 ;  
        end  
    else   
        output=0 ;    
    end  
else
    output=0 ;  
end
%ISODD Function to determine whether a number is odd
%
% Andy French. May 2003
%
% Syntax: OUTPUT=ISODD(INPUT)
%
% OUTPUT is 1 if input is an integer and is not divisible by 2 without any remainder
% OUTPUT is zero otherwise

%%

function output=iseven(input)
if isinteger(input)==1          %This is one of mine
    if rem(input,2)==0          %Remainder function - MATLAB standard
        output=0;
    else
        output=1;
    end
else
    output=0;
end

%%

%INTPERM Function that produces a matrix whose
% rows are the possible permutations of N integers
% varying from 1 to M.
%
% Andy French. September 2003.
%
% Syntax:
%
% OUTPUT=INTPERM(M,N)
%
% M, N are integers >=1
% OUTPUT is an M^N x N matrix.
%
% Example: output=intperm(4,3)

function output=intperm(m,n)
init=ones(m^n,n);
for k=1:m^n
    for j=1:n
        init(k,j)=ceil(mod(k/(m^(n-j)),m));
        if init(k,j)==0
            init(k,j)=m;
        end
    end
end

output=init;

%%

function output=ellipse(N,semi_maj,semi_min,tilt,centre)

if N>=8
    
    %Define ellipse in co-ordinates centred on ellipse centre
    %and aligned with SEMI_MAJ and SEMI_MIN axes
    
    angle_interval=2*pi/N ;
    
    for n=1:N
        angle=(n-1)*angle_interval;
        x=semi_maj*sine(angle);
        y=semi_min*cosine(angle);
        xy_vec=[x;y];
        
        %Rotate elements by tilt
        xy_vec=[cosine(tilt),sine(tilt);-sine(tilt),cosine(tilt)]*xy_vec;
        
        %Shift centre of ellipse
        col_vec_array(:,n)=xy_vec+[centre(1);centre(2)];
    end
    
    %Compute foci and eccentricity
    eccentricity=(1/semi_maj)*sqrt(semi_maj*semi_maj-semi_min*semi_min);
    foci_minus=centre-semi_maj*eccentricity*[cosine(tilt),-sine(tilt)];
    foci_plus=centre+semi_maj*eccentricity*[cosine(tilt),-sine(tilt)];
    
    %Define outputs
    output{1}=col_vec_array.' ;
    output{2}=[foci_minus;foci_plus];
    output{3}=eccentricity;
    
else
    
    error('Number of vertices in ellipse must be 8 or more')
end

%%

%ISODD Function to determine whether a number is odd
%
% Andy French. May 2003
%
% Syntax: OUTPUT=ISODD(INPUT)
%
% OUTPUT is 1 if input is an integer and is not divisible by 2 without any remainder
% OUTPUT is zero otherwise

function output=isodd(input)

if isinteger(input)==1          %This is one of mine
    if rem(input,2)==0          %Remainder function - MATLAB standard
        output=0;
    else
        output=1;
    end
else
    output=0;
end

%End of code.


