%pythagorean_triples
% Function which computes unique Pythagorean Triples, i.e. three integers
% a,b,c such that c^2 = a^2 + b^2
% 
% Note a,b,c are such that the triangles formed are not similar to any
% other right angles triangle whose sides are Pythagorean Triples.
% So 3,4,5 is a triple but 6,8,10 is not another possibility.
%
% LAST UPDATED by Andy French April 2016

function pythagorean_triples

%Display message to command window
disp(' ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%' )
disp('  Pythagorean Triple Generator ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%' )
disp(' ')

%Get parameter which generates Pythagorean triples
disp(' Define integer N which generates Pythagorean triples. ')
N = input(' > N = ');
if isempty(N)==1
    N = 30;
    disp(' Default N = 30.')
end

%

%Compute first N Pythagorean triples
[a,b,c,abc_sum,A,B,a_over_c,b_over_c] = pytrip( N, N );

%Display results to command window
disp(' ')
disp([' First ',num2str(length(a)),' Pythagorean triples are:'])
disp('           a           b           c'); 
disp([a,b,c])

%Write out data into an Excel spreadsheet
xls_cell = {['First ',num2str(length(a)),' Pythagorean triples'],'','','','','','','','';...
    'n','a','b','c','a + b + c','A /deg','B /deg','a/c','b/c'};
for n=1:length(a)
    xls_cell{2+n,1} = n;
    xls_cell{2+n,2} = a(n);
    xls_cell{2+n,3} = b(n);
    xls_cell{2+n,4} = c(n);
    xls_cell{2+n,5} = abc_sum(n);
    xls_cell{2+n,6} = A(n);
    xls_cell{2+n,7} = B(n);
    xls_cell{2+n,8} = a_over_c(n);
    xls_cell{2+n,9} = b_over_c(n);
end
xlswrite(['First ',num2str(length(a)),' Pythagorean triples'],xls_cell);
disp(' ')
disp(' Pythagorean triples computed and saved in an Excel file.')
disp(' ')

%%

%pytrip
% Function which computes Pythagorean Triples [a,b,c]
% a is the adjecent side of a raght angles triangle
% b is the opposite side of a right angled triangle
% c is the hypotenuse of a right angled triangle
%
% a,b,c are all integers.
%
% LAST UPDATED by Andy French September 2012
%
% Syntax: [a,b,c,abc_sum,A,B,a_over_c,b_over_c] = pytrip( Mmax, Kmax )
%
% Mmax and Kmax are positive integers which define the extent of triples
% computed. Approximately Kmax*(0.5Mmax^2 -Mmax) triples will be found.
% The triples will be stored in column vectors a,b,c and ordered by size of
% a+b+c
%
% A is the angle in degrees between the hypotenuse and the adjecent side
% B is the angle in degrees between the hypotenuse and the opposite side
%
% a_over_c and b_over_c are the adjecent and opposite sides scaled by the
% hypotenuse. There shall only be one incarnation of these pairs in the
% list, i.e. scaled triples with the same aspect ratio b/a will be removed.

function [a,b,c,abc_sum,A,B,a_over_c,b_over_c] = pytrip( Mmax, Kmax )

%Construct Mmax x Mmax x Kmax arrays of integers
m = ones(Mmax,Mmax,Kmax,'int32');
n = ones(Mmax,Mmax,Kmax,'int32');
k = ones(Mmax,Mmax,Kmax,'int32');
for j=1:Kmax
    m(:,:,j) = repmat( 1:Mmax, [Mmax,1] );
    n(:,:,j) = repmat( (1:Mmax).', [1,Mmax] );
    k(:,:,j) = j;
end

%Construct list of Pythagorean triples using Euclid's formula
a = k.*( m.^2 -n.^2 );
b = 2*m.*n.*k;
c = k.*( m.^2 + n.^2 );

%Reshape a,b,c arrays into column vectors
a = reshape( a, [numel(a),1] );
b = reshape( b, [numel(a),1] );
c = reshape( c, [numel(a),1] );

%Remove values when n >= m, i.e. when a <=0
bad = find( a<=0 );
a(bad) = []; b(bad) = []; c(bad) = [];

%Compute perimeter and internal angles
abc_sum = a+b+c;
A = (180/pi)*atan2(double(b),double(a));
B = 90 - A;

%Sort by triangle perimeter
[abc_sum,s] = sort(abc_sum,'ascend');
a = a(s); b = b(s); c = c(s); A = A(s); B = B(s);

%Remove multiple instances of angle A (i.e. similar triangles)
[A,s] = unique(A,'first');
a = a(s); b = b(s); c = c(s); abc_sum = abc_sum(s); B = B(s);

%Sort again by triangle perimeter
[abc_sum,s] = sort(abc_sum,'ascend');
a = a(s);
b = b(s);
c = c(s);
A = A(s);
B = B(s);

%When B is the same as A we have the same triangle rotated. Remove these
%multiple instances
s = [];
for q=1:length(A)
    %Find any instances of B which match the current A angle.
    %Make sure these duplicate values don't form part of the checking
    %process as these will remove the original A values
    if ismember(q,s)==0
        s = [find( abs(B-A(q))< 10^-10 ),s];
    end
end
a(s) = []; b(s) = []; c(s) = []; abc_sum(s) = []; A(s) = []; B(s) =[];

%Compute normalized sides
a_over_c = double(a)./double(c);
b_over_c = double(b)./double(c);

%End of code